unit UAStar;

interface
 uses UVoronoi, UUzelGeneratorHrana, Classes, Graphics, Dialogs;

 type
  TAStar = class(TObject)

   private
    FOdkud, FKam, FUzel: TUzel;
    FOpen, FClose, FCesta: TList;
    Voronoi: TVoronoiDiagram;

   public
    property Odkud: TUzel read FOdkud write FOdkud;
    property Kam: TUzel read FKam write FKam;
    property Cesta: TList read FCesta;

    constructor Create (aOdkud, aKam: TUzel);
    destructor Destroy; override;

    procedure Expanduj(aUzel: TUzel);
    procedure VykresliCestu;

    function NalezniCestu(aOdkud, aKam: TUzel): TList;
    function UzelNejlepsiOhodnoceni: TUzel;
    function Expandovan(aUzel: TUzel; aPole: Tlist): TUzel;
 end;

 var
   pos: Integer;

implementation

  uses MaskUtils, URozhrani;

  constructor TAStar.Create(aOdkud,aKam:Tuzel);
    begin
     Inherited Create;
      Odkud:= aOdkud;
      Kam:= aKam;
      FOpen:= TList.Create;
      FClose:= TList.Create;
      FCesta:= TList.Create;
      Voronoi:= TVoronoiDiagram.Create;
      Odkud.VKPrvnimu:= 0;
      Odkud.Ohodnoceni:= Voronoi.DejVzdalenost(Odkud,Kam)+Odkud.VKPrvnimu;
      NalezniCestu(aOdkud, aKam);
    end;

  destructor TAStar.Destroy;
    begin
      Voronoi.Free;
      FClose.Free;
      FOpen.Free;
      FCesta.Free;
      Odkud:=nil;
      Kam:=nil;
    Inherited Destroy;
    end;

function TAStar.NalezniCestu(aOdkud,aKam:TUzel):TList;
  Begin
    If (aOdkud=aKam) then
      Begin
       MessageDlg('Cesta nebyla nalezena',mtInformation,[mbOk],0);
       exit;
      end;
    FOpen.Add(aOdkud);
    FUzel:= UzelNejlepsiOhodnoceni;
    If Fuzel = nil then
      Begin
       MessageDlg('Cesta nenalezena',mtInformation,[mbOK],0);
       Exit;
      end;
    while FUzel <> nil do
      Begin
        If FUzel = Kam then
          Begin
            VykresliCestu;
            exit;
          end;
        Expanduj(FUzel);
        FClose.Add(FUzel);
        FOpen.Delete(pos);
        FUzel:= UzelNejlepsiOhodnoceni;
        If Fuzel = nil then MessageDlg('Cesta nenalezena',mtInformation,[mbOK],0);
      end;
    end;

function TAStar.UzelNejlepsiOhodnoceni: TUzel;
  var
    i: integer;
    Uzel: TUzel;
    Ohodnoceni: Single;
  Begin
    Ohodnoceni:= Nekonecno;
    Result:= nil;
    For i:=0 to FOpen.Count-1 do
    Begin
     Uzel:= TUzel(FOpen.Items[i]);
     If Uzel.Ohodnoceni < Ohodnoceni then
      Begin
        Ohodnoceni:= Uzel.Ohodnoceni;
        Result:= Uzel;
        pos:= i;
      end;
    end;
  end;

procedure TAStar.Expanduj(aUzel: Tuzel);
     var
      Uzel,Uzel1: TUzel;
      UzelVOpen,UzelVClose: TUzel;
      i: integer;
      Hrana: THrana;
     Begin
      Uzel:= aUzel;
      If Uzel = nil then exit;
      For i:=0 to Uzel.Hrany.Count-1 do
        Begin
          Hrana:= THrana(Uzel.Hrany.Items[i]);
        If Hrana.Uzel1 = Uzel then Uzel1:= Hrana.Uzel2
        else
        Uzel1:= Hrana.Uzel1;
        UzelVOpen:= Expandovan(Uzel1,FOpen);
        UzelVClose:= Expandovan(Uzel1,FClose);
        If UzelVOpen <> nil then
          Begin
            If UzelVOpen.Ohodnoceni > Uzel1.Ohodnoceni then
              Begin
                UzelVOpen.Ohodnoceni:= Uzel1.Ohodnoceni;
                UzelVOpen.Predchudce:= Uzel1.Predchudce;
                UzelVOpen.VKPrvnimu:= Uzel1.Predchudce.VKPrvnimu+Voronoi.DejVzdalenost(Uzel1,Uzel1.Predchudce);
              end;
          end
            else
              If UzelVClose = nil then
                Begin
                Uzel1.Predchudce:= Uzel;
                Uzel1.Ohodnoceni:=Uzel.VKPrvnimu+Voronoi.DejVzdalenost(Uzel,Uzel1)+Voronoi.DejVzdalenost(Uzel1,Kam);
                Uzel1.VKPrvnimu:=Uzel1.Predchudce.VKPrvnimu+Voronoi.DejVzdalenost(Uzel1,Uzel1.Predchudce);
                FOpen.Add(Uzel1);
                end;
            end;
          end;

function TAStar.Expandovan(aUzel: TUzel;aPole: TList): TUzel;
     var
     i: integer;
     UX,UY: Double;
  Begin
     Result:= nil;
     UX:=aUzel.X;
     UY:=aUzel.Y;
  For i:=0 to aPole.Count-1 do
    Begin
    If UX = TUzel(aPole.Items[i]).X then
    If UY = TUzel(aPole.Items[i]).Y then
    Result:=aPole.Items[i];
    end;
  end;

procedure TAStar.VykresliCestu;
  var
    i: integer;
    Uzel: TUzel;
  Begin
    Uzel:=Kam;
    while Uzel <> Odkud  do
      Begin
        Form1.PaintBox1.Canvas.Pen.Width:= 3;
        Form1.PaintBox1.Canvas.Pen.Color:= ClNavy;
        Cesta.Add(Uzel);
        Uzel:= Uzel.Predchudce;
      end;
  For i:=Cesta.Count-1 downto 0 do
    Begin
     Uzel:= TUzel(Cesta.Items[i]);
     If Uzel.Predchudce <> nil then
    Begin
     Form1.PaintBox1.Canvas.MoveTo(Round(Uzel.X),Round(Uzel.Y));
     Form1.PaintBox1.Canvas.LineTo(Round(Uzel.Predchudce.X),Round(Uzel.Predchudce.Y));
    end;
    end;
  end;
end.
