unit UVoronoi;

interface
  Uses UUzelGeneratorHrana, Classes, Graphics, QGraphics, Types, SysUtils,
      Dialogs, QComCtrls, ComCtrls;

  Type
   TVoronoiDiagram=Class(TObject)
    Private
     FSeznamVsechUzlu:TList;
     FSeznamVsechHran:TList;
     FSeznamVsechGeneratoru:TList;
     FStartBod:TUzel;
     FCilBod:TUzel;
     Function ReadPocet:Integer;
    Public
     Property PocetGeneratoru:Integer read ReadPocet;
     Property Generatory:TList read FSeznamVsechGeneratoru;
     Property Uzly:TList read FSeznamVsechUzlu write FSeznamVsechUzlu;
     Property StartBod:TUzel read FStartBod write FStartBod;
     Property CilBod:TUzel read FCilBod write FCilBod;

     Constructor Create;
     Destructor Destroy;override;

     Function PridejGenerator(aX,aY:Int64):TGenerator;
     Function ExistGenerator(aX,aY:Int64):Boolean;
     Function ExistPrusecik(aX,aY:Double;aPruseciky:Tlist):Boolean;
     Function DejVzdalenost(aObjekt1,aObjekt2:TObject):Double;
     Function NejblizsiGenerator(aGenerator:TGenerator):TGenerator;overload;
     Function NejblizsiGenerator(aUzel:Tuzel):TGenerator;overload;

     Function VytvorHranici(aGenerator1,aGenerator2:TGenerator):THrana; overload;
     Function VytvorHranici(aGenerator1,aGenerator2:TGenerator;aPrusecik:TPrusecik):THrana; overload;
     Function VytvorHranici(aNarOblast,aNovyGenerator:TGenerator;aBod,aBod2:TObject):THrana;overload;

     Function Blizsi(U1X,U1Y:Double;RodGenerator,NarGenerator:TGenerator):Boolean;overload;
     Function Blizsi(aU1X,aU1Y:Double;aP1,aP2:TPrusecik):Boolean;overload;
     Function NajdiNarusenouOblast(aPrusecik:TPrusecik;aNovyGenerator,aNarOblast:TGenerator):TGenerator;
     Function Neexistuje(aPrusecik:TPrusecik;aLSP:TList):Boolean;
     Function NajdiPrusecik(NovyGenerator,NarOblast:TGenerator;AktHrana:THrana;aBod:TPrusecik):TPrusecik;overload;
     Function NajdiPrusecik(aHranaVD:THrana;aUzel1,aUzel2:TUzel):TPrusecik;overload;
     Function NajdiDruhouHranu(aUzel:TUzel;aHrana:THrana;aStaryGenerator:TGenerator):THrana;

     Procedure VykresliGenerator(aGenerator:TGenerator);
     Procedure VykresliHranici(aHranice:THrana);
     Procedure VytvorPolygon(aGenerator:TGenerator);
     Procedure ZkratHranyVD(aGSP:TList;aGenerator:TGenerator);
     Procedure VymazHrany(aNovyGenerator:TGenerator;aProsle:TList);
     Procedure VymazHranu(aHrana:THrana);
     Procedure VymazUzel(aUzel:TUzel);
     Procedure VygumujHranu(aHrana:THrana);
     Procedure VymazUzly(aPozice:Integer);
     Procedure PrekresliHrany;
     Procedure PrekresliStartBod;
     Procedure PrekresliCilBod;
     Procedure PrekresliVsechnyGeneratory;
     Procedure ModifikujHranu(aHrana:THrana;aUzel:TUzel;aPrusecik:TPrusecik);overload;
     Procedure PropojBody;

     Procedure ZvolVhodny(aNovyPrusecik,aExistujiciPrusecik:TPrusecik;aNovyGenerator,aNarusenyGenerator:TGenerator);
   end;

var
 TloustkaCary:Integer;

Const
 PrumerGeneratoru=3;
 PrumerStartaKonc=6;
 KladnyOffsetUzlu=10000000;
 ZapornyOffsetUzlu=-10000000;
 Nekonecno=10000000;

implementation
  uses URozhrani, Math, ExtCtrls;

   Constructor TVoronoiDiagram.Create;
    Begin
     Inherited Create;
     FSeznamVsechUzlu:=TList.Create;
     FSeznamVsechHran:=TList.Create;
     FSeznamVsechGeneratoru:=TList.Create;
     StartBod:=nil;
     CilBod:=nil;
    end;

   Destructor TVoronoiDiagram.Destroy;
    var
     i:integer;
     Generator:TGenerator;
     Hrana:THrana;
     Uzel:TUzel;
    Begin
      For i:=0 to FSeznamVsechGeneratoru.Count-1 do
      Begin
      Generator:=TGenerator(FSeznamVsechGeneratoru.Items[i]);
       If Generator <> nil then
        Generator.Free;
      end;

      For i:=0 to FSeznamVsechHran.Count-1 do
      Begin
      Hrana:=THrana(FseznamVsechHran.Items[i]);
       If Hrana<>nil then
        Hrana.Free;
      end;

      For i:=0 to FSeznamVsechUzlu.Count-1 do
      Begin
      Uzel:=TUzel(FSeznamVsechUzlu.Items[i]);
       If Uzel<>nil then
        Uzel.Free;
      end;
      FSeznamVsechGeneratoru.Free;
      FSeznamVsechHran.Free;
      FSeznamVsechUzlu.Free;
     Inherited Destroy;
    end;

   Function TVoronoiDiagram.PridejGenerator(aX,aY:Int64):TGenerator;
    var
     Generator:TGenerator;
    Begin
     result:=nil;
     If ExistGenerator(aX,aY) = False then
      Begin
       Generator:=TGenerator.Create(aX,aY);
       FSeznamVsechGeneratoru.Add(Generator);
       VyKresliGenerator(Generator);
       result:=Generator;
      end;
    end;

   Function TVoronoiDiagram.ExistGenerator(aX,aY:Int64):Boolean;
    var
     i:integer;
     Begin
      result:=False;
       For i:=0 to FSeznamVsechGeneratoru.Count-1 do
        If (TGenerator(FSeznamVsechGeneratoru[i]).X = aX) and (TGenerator(FSeznamVsechGeneratoru[i]).Y = aY) then
         Result:=True;
     end;

   Procedure TVoronoiDiagram.VyKresliGenerator(aGenerator:TGenerator);
    var
     X,Y:Int64;
      Begin
       X:=aGenerator.X;
       Y:=aGenerator.Y;
       TloustkaCary:=2;
       Form1.PaintBox1.Canvas.Pen.Width:=TloustkaCary;
       Form1.PaintBox1.Canvas.Pen.Color:=ClBlack;
       Form1.PaintBox1.Canvas.Ellipse(X-PrumerGeneratoru,Y-PrumerGeneratoru,X+PrumerGeneratoru,Y+PrumerGeneratoru)
      end;

  Function TVoronoiDiagram.DejVzdalenost(aObjekt1,aObjekt2:TObject):Double;
   var
    X1,X2,Y1,Y2,RozdilX,RozdilY:Double;
    Xnadruhou,Ynadruhou:Double;
   Begin
   Result:=0;
   X1:=0;
   X2:=0;
   Y1:=0;
   Y2:=0;
   RozdilX:=0;
   RozdilY:=0;
   If (aObjekt1 is TGenerator) and (aObjekt2 is TGenerator) then
    Begin
    X1:=TGenerator(aObjekt1).X;
    X2:=TGenerator(aObjekt2).X;
    Y1:=TGenerator(aObjekt1).Y;
    Y2:=TGenerator(aObjekt2).Y;
    end;

   If (aObjekt1 is TGenerator) and (aObjekt2 is TUzel) then
    Begin
    X1:=TGenerator(aObjekt1).X;
    X2:=TUzel(aObjekt2).X;
    Y1:=TGenerator(aObjekt1).Y;
    Y2:=TUzel(aObjekt2).Y;
    end;

   If (aObjekt1 is TPrusecik) and (aObjekt2 is TUzel) then
    Begin
    X1:=TPrusecik(aObjekt1).X;
    X2:=TGenerator(aObjekt2).X;
    Y1:=TPrusecik(aObjekt1).Y;
    Y2:=TGenerator(aObjekt2).Y;
    end;

   If (aObjekt1 is TUzel) and (aObjekt2 is TUzel) then
   Begin
   X1:=TUzel(aObjekt1).X;
   X2:=TUzel(aObjekt2).X;
   Y1:=TUzel(aObjekt1).Y;
   Y2:=TUzel(aObjekt2).Y;
   end;
    RozdilX:=X2-X1;
    RozdilY:=Y2-Y1;
    Xnadruhou:=Power(RozdilX,2);
    Ynadruhou:=Power(RozdilY,2);
    Result:= Power((Xnadruhou+Ynadruhou),0.5);
   end;

  Function TVoronoiDiagram.NejblizsiGenerator(aGenerator:TGenerator):TGenerator;
    var
     NovyGenerator,NejblizsikNemu:TGenerator;
     i:Integer;
     Vzdalenost,NovaVzdalenost:Double;
      Begin
      NovyGenerator:=aGenerator;
      NejblizsikNemu:=nil;
      Vzdalenost:=Nekonecno;

      If FSeznamVsechGeneratoru.Count >=2 then
         For i:=0 to FSeznamVsechGeneratoru.Count-1 do
           Begin
           NovaVzdalenost:=DejVzdalenost(NovyGenerator,TGenerator(FSeznamVsechGeneratoru[i]));
           If (NovaVzdalenost < Vzdalenost) and (NovyGenerator <> TGenerator(FSeznamVsechGeneratoru[i])) then
           Begin
           Vzdalenost:=NovaVzdalenost;
           NejblizsikNemu:=TGenerator(FSeznamVsechGeneratoru[i]);
           end;
         end;
           Result:=NejblizsikNemu;
      end;

  Function TVoronoiDiagram.NejblizsiGenerator(aUzel:TUzel):TGenerator;
    var
     NejblizsikNemu:TGenerator;
     Uzel:TUzel;
     UX,UY:Double;
     i:Integer;
     Vzdalenost,NovaVzdalenost:Double;
      Begin
      Uzel:=aUzel;
      NejblizsikNemu:=nil;
      Vzdalenost:=Nekonecno;

      If FSeznamVsechGeneratoru.Count >=2 then
         For i:=0 to FSeznamVsechGeneratoru.Count-1 do
           Begin
           NovaVzdalenost:=DejVzdalenost(TGenerator(FSeznamVsechGeneratoru[i]),Uzel);
           If (NovaVzdalenost < Vzdalenost)then
           Begin
           Vzdalenost:=NovaVzdalenost;
           NejblizsikNemu:=TGenerator(FSeznamVsechGeneratoru[i]);
           end;
         end;
           Result:=NejblizsikNemu;
      end;

  Function TVoronoiDiagram.Blizsi(U1X,U1Y:Double;RodGenerator,NarGenerator:TGenerator):Boolean;
   var
    RX,RY:Double;
    RodVzd,NarVzd,Vysl:Double;
    RX2,RY2,Soucet:Double;
    Begin
    Result:=False;
    RX:=RodGenerator.X-U1X;
    RY:=RodGenerator.Y-U1Y;
    RX2:= Power(RX,2);
    RY2:= Power(RY,2);
    Soucet:=RX2+RY2;
    RodVzd:= Power(Soucet,0.5);

    RX:=NarGenerator.X-U1X;
    RY:=NarGenerator.Y-U1Y;
    RX2:= Power(RX,2);
    RY2:= Power(RY,2);
    Soucet:=RX2+RY2;
    NarVzd:= Power(Soucet,0.5);

    Vysl:= Min(RodVzd,NarVzd);

    If Vysl = RodVzd then
    Result:=False;
    If Vysl = NarVzd then
    Result:=True;
    end;

   Function TVoronoiDiagram.VytvorHranici(aGenerator1,aGenerator2:TGenerator):THrana;
      var
       K1,Kk1,q1:Double;
       SX,SY,U1X,U1Y,U2X,U2Y:Double;
       Uz1X,Uz2X,Uz1Y,Uz2Y:Int64;
       i:Integer;
       U1,U2:TUzel;
       Hrana:THrana;
       G1,G2:TGenerator;
         Begin
         U1:=nil;
         U2:=nil;
         Result:=nil;

         G1:=aGenerator1;
         G2:=aGenerator2;
         If (G1.X = G2.X) and (G1.Y = G2.Y) then exit;

         If G1.X = G2.X then
          Begin
          Kk1:= Nekonecno;
          K1:= 0;
          end;

          If G1.Y = G2.Y then
          Begin
          Kk1:=0;
          K1:= Nekonecno;
          end;

          If G1.X <> G2.X then
          If G1.Y <> G2.Y then
          Begin
          Kk1:= (G2.Y-G1.Y)/(G2.X-G1.X);
          K1:=-1/Kk1;
          end;

          SX:=((G2.X-G1.X)/2)+G1.X;
          SY:=((G2.Y-G1.Y)/2)+G1.Y;

          If K1 <> Nekonecno then
          Begin
          q1:=SY-(K1*SX);
          U1X:=KladnyOffsetUzlu+SX;
          U1Y:=K1*U1X+q1;
          U2X:=ZapornyOffsetUzlu+SX;
          U2Y:=K1*U2X+q1;
          end
          else
          Begin
          U1X:=((G2.X-G1.X)/2)+G1.X;
          U2X:=U1X;
          U1Y:=KladnyOffsetUzlu+SY;
          U2Y:=ZapornyOffsetUzlu+SY;
          end;

         For i:=0 to FSeznamVsechUzlu.Count-1 do
         Begin
          If (TUzel(FSeznamVsechUzlu.Items[i]).X = U1X) and (TUzel(FSeznamVsechUzlu.Items[i]).Y = U1Y) then
            U1:=TUzel(FSeznamVsechUzlu.Items[i]);

          If (TUzel(FSeznamVsechUzlu.Items[i]).X = U2X) and (TUzel(FSeznamVsechUzlu.Items[i]).Y = U2Y) then
            U2:=TUzel(FSeznamVsechUzlu.Items[i]);
         end;

          If U1 = nil then
          Begin
          U1:=TUzel.Create(U1X,U1Y);
          FSeznamVsechUzlu.Add(U1);
          end;

          If U2 = nil then
          Begin
          U2:=TUzel.Create(U2X,U2Y);
          FSeznamVsechUzlu.Add(U2)
          end;

         For i:=0 to FSeznamVsechHran.Count-1 do
           Begin
           Hrana:=THrana(FSeznamVsechHran.Items[i]);
           If ((U1 = Hrana.Uzel1) and (U2 = Hrana.Uzel2)) or ((U1 = Hrana.Uzel2) and (U2 = Hrana.Uzel1)) then
           If ((G1 = Hrana.Generator1) and (G2 = Hrana.Generator2)) or ((G1 = Hrana.Generator2) and (G2 = Hrana.Generator1)) then
           Begin
           Result:= FSeznamVsechHran.Items[i];
           exit;
           end;
           end;

          Hrana:=THrana.Create(U1,U2,G1,G2);
          aGenerator1.Hrany.Add(Hrana);
          aGenerator2.Hrany.Add(Hrana);
          U1.Hrany.Add(Hrana);
          U2.Hrany.Add(Hrana);
          FSeznamVsechHran.Add(Hrana);
          Result:=Hrana;
         end;

  Function TVoronoiDiagram.VytvorHranici(aGenerator1,aGenerator2:TGenerator;aPrusecik:TPrusecik):THrana;
      var
       K1,Kk1,q1:Double;
       SX,SY,U1X,U1Y,U2X,U2Y:Double;
       i:Integer;
       Uz1X,Uz2X,Uz1Y,Uz2Y:Double;
       U1,U2:TUzel;
       Hrana:THrana;
       NG,G1,G2,RG:TGenerator;
         Begin
         U1:=nil;
         U2:=nil;
         Result:=nil;
         G1:=aGenerator1;
         G2:=aGenerator2;

          If G1.X = G2.X then
          Begin
          Kk1:= Nekonecno;
          K1:= 0;
          end;

          If G1.Y = G2.Y then
          Begin
          Kk1:=0;
          K1:= Nekonecno;
          end;

          If G1.X <> G2.X then
          If G1.Y <> G2.Y then
          Begin
          Kk1:= (G2.Y-G1.Y)/(G2.X-G1.X);
          K1:=-1/Kk1;
          end;

          SX:=((G2.X-G1.X)/2)+G1.X;
          SY:=((G2.Y-G1.Y)/2)+G1.Y;

          SX:=aPrusecik.X;
          SY:=aPrusecik.Y;

          If K1 <> Nekonecno then
          Begin
          q1:=SY-(K1*SX);
          U1X:=KladnyOffsetUzlu+SX;
          U1Y:=K1*U1X+q1;
          U2X:=ZapornyOffsetUzlu+SX;
          U2Y:=K1*U2X+q1;
          end
          else
          Begin
          U1X:=((G2.X-G1.X)/2)+G1.X;
          U2X:=U1X;
          U1Y:=KladnyOffsetUzlu+SY;
          U2Y:=ZapornyOffsetUzlu+SY;
          end;

          NG:=NajdiNarusenouOblast(aPrusecik,G1,G2);

          If Blizsi(U1X,U1Y,G1,NG) then
          Begin
          U1X:=aPrusecik.X;
          U1Y:=aPrusecik.Y;
          end
          else
          Begin
          U2X:=aPrusecik.X;
          U2Y:=aPrusecik.Y;
          end;

         For i:=0 to FSeznamVsechUzlu.Count-1 do
         Begin
          If (TUzel(FSeznamVsechUzlu.Items[i]).X = U1X) and (TUzel(FSeznamVsechUzlu.Items[i]).Y = U1Y) then
          Begin
          U1:=TUzel(FSeznamVsechUzlu.Items[i]);
          end;

          If (TUzel(FSeznamVsechUzlu.Items[i]).X = U2X) and (TUzel(FSeznamVsechUzlu.Items[i]).Y = U2Y) then
          Begin
          U2:=TUzel(FSeznamVsechUzlu.Items[i]);
          end;
         end;

          If U1 = nil then
          Begin
          U1:=TUzel.Create(U1X,U1Y);
          FSeznamVsechUzlu.Add(U1);
          end;

          If U2 = nil then
          Begin
          U2:=TUzel.Create(U2X,U2Y);
          FSeznamVsechUzlu.Add(U2)
          end;

         For i:=0 to FSeznamVsechHran.Count-1 do
           Begin
           Hrana:=THrana(FSeznamVsechHran.Items[i]);
           If ((U1 = Hrana.Uzel1) and (U2 = Hrana.Uzel2)) or ((U1 = Hrana.Uzel2) and (U2 = Hrana.Uzel1)) then
           If ((G1 = Hrana.Generator1) and (G2 = Hrana.Generator2)) or ((G1 = Hrana.Generator2) and (G2 = Hrana.Generator1)) then
           Begin
           Result:= FSeznamVsechHran.Items[i];
           exit;
           end;
           end;

          Hrana:=THrana.Create(U1,U2,G1,G2);
          aGenerator1.Hrany.Add(Hrana);
          aGenerator2.Hrany.Add(Hrana);
          U1.Hrany.Add(Hrana);
          U2.Hrany.Add(Hrana);
          FSeznamVsechHran.Add(Hrana);
          Result:=Hrana;
          end;

   Function TVoronoiDiagram.VytvorHranici(aNarOblast,aNovyGenerator:TGenerator;aBod:TObject;aBod2:TObject):THrana;
   var
    U1X,U1Y,U2X,U2Y:Double;
    i:Integer;
    Uzel1,Uzel2:TUzel;
    Hrana:THrana;
     Begin

     If (aBod is TPrusecik) then
     Begin
     U1X:=TPrusecik(aBod).X;
     U1Y:=TPrusecik(aBod).Y;
     end;

     If (aBod is TUzel) then
     Begin
     U1X:=TUzel(aBod).X;
     U1Y:=TUzel(aBod).Y;
     end;

     If (aBod is TPrusecik) then
     Begin
     U1X:=TPrusecik(aBod).X;
     U1Y:=TPrusecik(aBod).Y;
     end;

     If (aBod2 is TUzel) then
     Begin
     U2X:=TUzel(aBod2).X;
     U2Y:=TUzel(aBod2).Y;
     end;

     Uzel1:=nil;
     Uzel2:=nil;

      For i:=0 to FSeznamVsechUzlu.Count-1 do
      Begin
          If (TUzel(FSeznamVsechUzlu.Items[i]).X = U1X) and (TUzel(FSeznamVsechUzlu.Items[i]).Y = U1Y) then
            Uzel1:=TUzel(FSeznamVsechUzlu.Items[i]);

          If (TUzel(FSeznamVsechUzlu.Items[i]).X = U2X) and (TUzel(FSeznamVsechUzlu.Items[i]).Y = U2Y) then
            Uzel2:=TUzel(FSeznamVsechUzlu.Items[i]);
      end;

          If Uzel1 = nil then
          Begin
          Uzel1:=TUzel.Create(U1X,U1Y);
          FSeznamVsechUzlu.Add(Uzel1);
          end;

          If Uzel2 = nil then
          Begin
          Uzel2:=TUzel.Create(U2X,U2Y);
          FSeznamVsechUzlu.Add(Uzel2)
          end;

      For i:=0 to FSeznamVsechHran.Count-1 do
      Begin
           Hrana:=THrana(FSeznamVsechHran.Items[i]);
           If ((Uzel1 = Hrana.Uzel1) and (Uzel2 = Hrana.Uzel2)) or ((Uzel1 = Hrana.Uzel2) and (Uzel2 = Hrana.Uzel1)) then
           If ((aNovyGenerator = Hrana.Generator1) and (aNarOblast = Hrana.Generator2)) or ((aNovyGenerator = Hrana.Generator2) and (aNarOblast = Hrana.Generator1)) then
           Begin
           Result:= FSeznamVsechHran.Items[i];
           exit;
           end;
      end;

          Hrana:=THrana.Create(Uzel1,Uzel2,aNovyGenerator,aNarOblast);
          If aNarOblast <> nil then
          aNarOblast.Hrany.Add(Hrana);
          If aNovyGenerator <> nil then
          aNovyGenerator.Hrany.Add(Hrana);
          Uzel1.Hrany.Add(Hrana);
          Uzel2.Hrany.Add(Hrana);
          FSeznamVsechHran.Add(Hrana);
          Result:=Hrana;
     end;

   Function TVoronoiDiagram.NajdiNarusenouOblast(aPrusecik:TPrusecik;aNovyGenerator,aNarOblast:TGenerator):TGenerator;
   var
   NarusHrana:THrana;
   G1,G2,NarusGenerator:TGenerator;
   Generator1,Generator2:TGenerator;
   Begin
   NarusHrana:=aPrusecik.Hrana;
   G1:=NarusHrana.Generator1;
   G2:=NarusHrana.Generator2;
   Generator1:=aNovyGenerator;
   Generator2:=aNarOblast;
   result:=nil;

   If (G1 = Generator1) or (G1 = Generator2) then
   NarusGenerator:= G2
   else
   NarusGenerator:=G1;

   result:=NarusGenerator;
   end;

   Procedure TVoronoiDiagram.VykresliHranici(aHranice:THrana);
      Begin
      Form1.PaintBox1.Canvas.Pen.Width:=1;
      Form1.PaintBox1.Canvas.Pen.Style:=Graphics.psSolid;
      Form1.PaintBox1.Canvas.Pen.Color:=ClBlack;
      Form1.PaintBox1.Canvas.MoveTo(Round(aHranice.Uzel1.X),Round(aHranice.Uzel1.Y));
      Form1.PaintBox1.Canvas.LineTo(Round(aHranice.Uzel2.X),Round(aHranice.Uzel2.Y));
      end;

    Procedure TVoronoiDiagram.VytvorPolygon(aGenerator:TGenerator);
     var
      GSP,LSP,ProsleOblasti:TList;
      NarOblast,NovyGenerator,Prvni,Predposledni:TGenerator;
      Prusecik,ZakoncBod,aBod:TPrusecik;
      Hranice,AktHrana:THrana;
      PHNG,i,j,Pozice:Integer;
      Begin

      NarOblast:=nil;
      Prusecik:=nil;
      ZakoncBod:=nil;
      Hranice:=nil;
        Predposledni:=nil;

      If FSeznamVsechGeneratoru.Count < 2 then exit;
        aBod:=nil;
        NovyGenerator:=aGenerator;
        Prvni:= NejblizsiGenerator(NovyGenerator);
        NarOblast:=Prvni;
        GSP:=TList.Create;
        LSP:=TList.Create;
        ProsleOblasti:=Tlist.Create;
     If NovyGenerator = nil then exit;

     While NarOblast <> nil do
       Begin
        PHNG:= NarOblast.Hrany.Count;
        For i:=0 to PHNG-1 do
            Begin
            AktHrana:=THrana(NarOblast.Hrany.Items[i]);
            Prusecik:=NajdiPrusecik(NovyGenerator,NarOblast,AktHrana,aBod);
            If Prusecik <> nil then
            If Neexistuje(Prusecik,LSP) then
              Begin
              If aBod <> nil then
              If aBod.Hrana <> Prusecik.Hrana then
                Begin
                LSP.Add(Prusecik);
                GSP.Add(Prusecik);
                end
                else
                Prusecik.Free;
              If aBod = nil then
                Begin
                LSP.Add(Prusecik);
                GSP.Add(Prusecik);
                end;
              end
              else
              Prusecik.Free;
            end;

       If LSP.Count = 0 then
       Begin
       If aBod = nil then
       Hranice:=VytvorHranici(NovyGenerator,NarOblast);
       If aBod <> nil then
       Hranice:=VytvorHranici(NovyGenerator,NarOblast,aBod);
       NarOblast:=nil;
       aBod:=nil;
       end;

       If LSP.Count = 1 then
       Begin
       If aBod<> nil then
       Begin
       Hranice:=VytvorHranici(NarOblast,NovyGenerator,aBod,TPrusecik(LSP.Items[0]));
       aBod:=TPrusecik(LSP.Items[0]);
       NarOblast:=NajdiNarusenouOblast(aBod,NovyGenerator,NarOblast);
       end;
       If aBod = nil then
       Begin
       Hranice:=VytvorHranici(NarOblast,NovyGenerator,TPrusecik(LSP.Items[0]));
       aBod:=TPrusecik(LSP.Items[0]);
       NarOblast:=NajdiNarusenouOblast(TPrusecik(LSP.Items[0]),NovyGenerator,NarOblast);
       end;
       If (NarOblast = Predposledni) or (NarOblast = NovyGenerator)  then
       Begin
       If (ZakoncBod <> nil) then
       Hranice:=VytvorHranici(NovyGenerator,NarOblast,aBod,ZakoncBod)
       else
       Hranice:=VytvorHranici(NovyGenerator,NarOblast,aBod);
       NarOblast:=nil;
       aBod:=nil;
       end;
       end;

       If LSP.Count = 2 then
       Begin
       ZakoncBod:=TPrusecik(LSP.Items[0]);
       Predposledni:=NajdiNarusenouOblast(ZakoncBod,NovyGenerator,NarOblast);
       Prusecik:=TPrusecik(LSP.Items[1]);
       Hranice:=VytvorHranici(NarOblast,NovyGenerator,ZakoncBod,Prusecik);
       aBod:=Prusecik;
       NarOblast:=NajdiNarusenouOblast(aBod,NovyGenerator,NarOblast);
       If (NarOblast = Predposledni) or (NarOblast = NovyGenerator) then
       Begin
       Hranice:=VytvorHranici(NovyGenerator,NarOblast,aBod,ZakoncBod);
       NarOblast:=nil;
       aBod:=nil;
       end;
       end;

       If LSP.Count > 2 then
       Begin
       MessageDlg('Chyba,nasel jsem vice pruseciku',mtWarning,[mbOK],0);
       NarOblast:=nil;
       aBod:=nil;
       end;
       ProsleOblasti.Add(NarOblast);
       LSP.Clear;
     end;

     If GSP.Count <> 0 then
     ZkratHranyVD(GSP,NovyGenerator);
     VymazHrany(NovyGenerator,ProsleOblasti);
     VymazUzly(0);
     PrekresliHrany;
     PrekresliVsechnyGeneratory;

     For i:=0 to GSP.Count-1 do
     If TPrusecik(GSP.Items[i]) <> nil then
     TPrusecik(GSP.Items[i]).Free;

     ProsleOblasti.Free;
     GSP.Free;
     LSP.Free;
   end;


    Function TVoronoiDiagram.NajdiPrusecik(NovyGenerator,NarOblast:TGenerator;AktHrana:THrana;aBod:TPrusecik):TPrusecik;
    var
    i,j,U2X,U2Y:Integer;
    K1,K2,Kk1,q1,q2,q,SX,SY,dY,dX:Double;
    VSU1,VSU2,VNU1,VNU2:Double;
    PX,PY:Double;
    Hrana:THrana;
    MinX,MaxX,MinY,MaxY:Double;
    Begin
    Result:=nil;
    If (NovyGenerator.X = NarOblast.X) and (NovyGenerator.Y = NarOblast.Y) then exit;
    Hrana:=AktHrana;


     If NovyGenerator.X = NarOblast.X then
     Begin
     Kk1:= Nekonecno;
     K1:= 0;
     end;

     If NovyGenerator.Y = NarOblast.Y then
     Begin
     Kk1:=0;
     K1:= Nekonecno;
     end;

     If NovyGenerator.X <> NarOblast.X then
     If NovyGenerator.Y <> NarOblast.Y then
     Begin
     Kk1:= (NarOblast.Y-NovyGenerator.Y)/(NarOblast.X-NovyGenerator.X);
     K1:=-1/Kk1;
     end;

     If Hrana.Uzel2.X <> Hrana.Uzel1.X then
     K2:= (Hrana.Uzel2.Y - Hrana.Uzel1.Y) / (Hrana.Uzel2.X - Hrana.Uzel1.X)
     else
     K2:= Nekonecno;

     SX:=((NarOblast.X-NovyGenerator.X)/2)+NovyGenerator.X;
     SY:=((NarOblast.Y-NovyGenerator.Y)/2)+NovyGenerator.Y;

     If aBod <> nil then
     Begin
     SX:=aBod.X;
     SY:=aBod.Y;
     end;

     q1:=SY-(K1*SX);

     SX:=((Hrana.Uzel2.X-Hrana.Uzel1.X)/2)+Hrana.Uzel1.X;
     SY:=((Hrana.Uzel2.Y-Hrana.Uzel1.Y)/2)+Hrana.Uzel1.Y;

     q2:=SY-(K2*SX);

     If K1 = K2 then
     Begin
     Result:=nil;
     exit;
     end;

     If K1 <> K2 then
     If (K1 <> Nekonecno) and (K2 <> Nekonecno) then
     Begin
     PX:=(q2-q1)/(K1-K2);
     PY:=(K1*PX)+q1;
     end;

     If (K1 = nekonecno) and (K1 <> K2) then
     Begin
     PX:=((NarOblast.X-NovyGenerator.X)/2)+NovyGenerator.X;
     PY:=(K2*PX)+q2;
     end;

     If (K2 = Nekonecno) and (K1 <> K2) then
     Begin
     PX:=Hrana.Uzel1.X;
     PY:=(K1*PX)+q1;
     end;

     If ((PX >= (Hrana.Uzel1.X-0.00001)) and (PX <= (Hrana.Uzel1.X+0.00001))) and ((PY >= (Hrana.Uzel1.Y-0.00001)) and (PY <= (Hrana.Uzel1.Y+0.00001))) then
     Begin
     VSU1:= DejVzdalenost(NarOblast,Hrana.Uzel1);
     VSU2:= DejVzdalenost(NarOblast,Hrana.Uzel2);
     VNU1:= DejVzdalenost(NovyGenerator,Hrana.Uzel1);
     VNU2:= DejVzdalenost(NovyGenerator,Hrana.Uzel2);

     If (VNU1+VNU2) < (VSU1+VSU2) then
     Begin
     Result:=TPrusecik.Create(PX,PY,Hrana);
     exit;
     end
     else
     Begin
     Result:=nil;
     exit;
     end;
     end;

     If ((PX >= (Hrana.Uzel2.X-0.00001)) and (PX <= (Hrana.Uzel2.X+0.00001))) and ((PY >= (Hrana.Uzel2.Y-0.00001)) and (PY <= (Hrana.Uzel2.Y+0.00001))) then
     Begin
     VSU1:= DejVzdalenost(NarOblast,Hrana.Uzel1);
     VSU2:= DejVzdalenost(NarOblast,Hrana.Uzel2);
     VNU1:= DejVzdalenost(NovyGenerator,Hrana.Uzel1);
     VNU2:= DejVzdalenost(NovyGenerator,Hrana.Uzel2);

     If (VNU1+VNU2) < (VSU1+VSU2) then
     Begin
     Result:=TPrusecik.Create(PX,PY,Hrana);
     exit;
     end
     else
     Begin
     Result:=nil;
     exit;
     end;
     end;

     MaxX:=Max(Hrana.Uzel1.X,Hrana.Uzel2.X);
     MinX:=Min(Hrana.Uzel1.X,Hrana.Uzel2.X);
     MaxY:=Max(Hrana.Uzel1.Y,Hrana.Uzel2.Y);
     MinY:=Min(Hrana.Uzel1.Y,Hrana.Uzel2.Y);

     If K2 = 0 then
     If (PX > MinX) and (PX < MaxX) then
     Begin
     Result:=TPrusecik.Create(PX,PY,Hrana);
     end;

     If K2 = Nekonecno then
     If (PY > MinY) and (PY < MaxY) then
     Begin
     Result:=TPrusecik.Create(PX,PY,Hrana);
     end;

     If (K2 <> 0) and (K2 <> Nekonecno) then
     If (PX >= MinX) and (PX <= MaxX) and (PY >= MinY) and (PY <= MaxY) then
     Begin
     Result:=TPrusecik.Create(PX,PY,Hrana);
     end;
end;

    Procedure TVoronoiDiagram.ZkratHranyVD(aGSP:TList;aGenerator:TGenerator);
    var
    Prusecik:TPrusecik;
    Hrana:THrana;
    G1,G2:TGenerator;
    U1,U2:TUzel;
    NovyGenerator:TGenerator;
    VU1,VU2,VRU1,VRU2:Double;
    i,j:Integer;
     Begin
     NovyGenerator:=aGenerator;

       For i:=0 to aGSP.Count-1 do
        Begin
        Prusecik:=TPrusecik(aGSP.Items[i]);
        Hrana:=Prusecik.Hrana;
        G1:=Hrana.Generator1;
        G2:=Hrana.Generator2;
        U1:=Hrana.Uzel1;
        U2:=Hrana.Uzel2;
        VU1:=DejVzdalenost(NovyGenerator,U1);
        VU2:=DejVzdalenost(NovyGenerator,U2);
        VRU1:=Min(DejVzdalenost(G1,U1),DejVzdalenost(G2,U1));
        VRU2:=Min(DejVzdalenost(G1,U2),DejVzdalenost(G2,U2));

        If (VU1 < VRU1) and (VU2 < VRU2) then
        Begin
        Hrana:=nil;
        end;

        If VU1 < VRU1 then
        If Hrana <> nil then
        Begin
        ModifikujHranu(Hrana,U2,Prusecik);
        end;

        If VU2 < VRU2 then
        If Hrana <> nil then
        Begin
        ModifikujHranu(Hrana,U1,Prusecik);
        end;
     end;
     end;

    Procedure TVoronoiDiagram.VymazHranu(aHrana:THrana);
    Var
    Hrana:THrana;
    G1,G2:TGenerator;
    U1,U2:TUzel;
    i,j:Integer;
    Begin
    Hrana:=aHrana;
    G1:=Hrana.Generator1;
    G2:=Hrana.Generator2;
    U1:=Hrana.Uzel1;
    U2:=Hrana.Uzel2;

    If G1<>nil then
    Begin
      For i:=0 to G1.Hrany.Count-1 do
      If Hrana = THrana(G1.Hrany.Items[i]) then
      Begin
      G1.Hrany.Delete(i);
      Break;
      end;
    end;

    If G2<>nil then
    Begin
      For i:=0 to G2.Hrany.Count-1 do
      If Hrana = THrana(G2.Hrany.Items[i]) then
      Begin
      G2.Hrany.Delete(i);
      Break;
      end;
    end;

    If U1<> nil then
    Begin
      For i:=0 to U1.Hrany.Count-1 do
      If Hrana = THrana(U1.Hrany.Items[i]) then
      Begin
      U1.Hrany.Delete(i);
      Break;
      end;
    end;

    If U2<> nil then
    Begin
      For i:=0 to U2.Hrany.Count-1 do
      If Hrana = THrana(U2.Hrany.Items[i]) then
      Begin
      U2.Hrany.Delete(i);
      Break;
      end;
    end;

   If Hrana <> nil then
   Begin
    For i:=0 to FSeznamVsechHran.Count-1 do
    If Hrana = THrana(FSeznamVsechHran.Items[i]) then
    Begin
    FSeznamVsechHran.Delete(i);
    Break;
    end;
   end;

   VygumujHranu(Hrana);
   Hrana.Free;

    If U1.Hrany.Count = 0 then
    VymazUzel(U1);

    If U2.Hrany.Count = 0 then
    VymazUzel(U2);

    end;

    Procedure TVoronoiDiagram.VymazHrany(aNovyGenerator:TGenerator;aProsle:TList);
    var
    Hrana:THrana;
    G,G1,G2,NovyGenerator:TGenerator;
    U1,U2:TUzel;
    i,j,k:Integer;
    VU1,VU2,VRU1,VRU2:Double;
    Begin
    NovyGenerator:=aNovyGenerator;

     For i:=aProsle.Count-1 downto 0 do
     Begin
     G:=TGenerator(aProsle.Items[i]);
     If G <> nil then
     Begin
     For j:=G.Hrany.Count-1 downto 0 do
     Begin
     Hrana:=THrana(G.Hrany.Items[j]);
     G1:=Hrana.Generator1;
     G2:=Hrana.Generator2;
     U1:=Hrana.Uzel1;
     U2:=Hrana.Uzel2;
     VRU1:=DejVzdalenost(G1,U1);
     VU1:=DejVzdalenost(NovyGenerator,U1);
     VRU2:=DejVzdalenost(G1,U2);
     VU2:=DejVzdalenost(NovyGenerator,U2);


     If ((VU1 < VRU1) and (VU2 < VRU2))  then
     If U1 <> U2 then
     If (G1 <> NovyGenerator) and (G2 <> NovyGenerator) then
     Begin
     VygumujHranu(Hrana);
      For k:=0 to U1.Hrany.Count-1 do
      If Hrana = THrana(U1.Hrany.Items[k]) then
      Begin
      U1.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to U2.Hrany.Count-1 do
      If Hrana = THrana(U2.Hrany.Items[k]) then
      Begin
      U2.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to G1.Hrany.Count-1 do
      If Hrana = THrana(G1.Hrany.Items[k]) then
      Begin
      G1.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to G2.Hrany.Count-1 do
      If Hrana = THrana(G2.Hrany.Items[k]) then
      Begin
      G2.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to FSeznamVsechHran.Count-1 do
      If Hrana = THrana(FSeznamVsechHran.Items[k]) then
      Begin
      FSeznamVsechHran.Delete(k);
      break;
      end;


     end;

    If (U1.X = U2.X) and (U1.Y = U2.Y) then
    Begin

      For k:=0 to U1.Hrany.Count-1 do
      If Hrana = THrana(U1.Hrany.Items[k]) then
      Begin
      U1.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to U1.Hrany.Count-1 do
      If Hrana = THrana(U2.Hrany.Items[k]) then
      Begin
      U2.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to G1.Hrany.Count-1 do
      If Hrana = THrana(G1.Hrany.Items[k]) then
      Begin
      G1.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to G2.Hrany.Count-1 do
      If Hrana = THrana(G2.Hrany.Items[k]) then
      Begin
      G2.Hrany.Delete(k);
      Break;
      end;

      For k:=0 to FSeznamVsechHran.Count-1 do
      If Hrana = THrana(FSeznamVsechHran.Items[k]) then
      Begin
      FSeznamVsechHran.Delete(k);
      break;
      end;

     end;
    end;
   end;
  end;
 end;

  Procedure TVoronoiDiagram.VymazUzel(aUzel:TUzel);
  var
  Uzel:TUzel;
  i:integer;
  Begin
  Uzel:=aUzel;
  If Uzel <> nil then
  Begin
    For i:=0 to FSeznamVsechUzlu.Count-1 do
    If Uzel = TUzel(FSeznamVsechUzlu.Items[i]) then
    Begin
    FSeznamVsechUzlu.Delete(i);
    Break;
    end;
  Uzel.Free;
  end;
  end;

  Procedure TVoronoiDiagram.VymazUzly(aPozice:Integer);
  var
  Uzel:TUzel;
  i,Pozice:integer;
  Znovu:Boolean;
   Begin
   Znovu:=False;
   Pozice:=aPozice;
   If Pozice >= FSeznamVsechUzlu.Count then exit;
   For i:=Pozice to FSeznamVsechUzlu.Count-1 do
   Begin
    Uzel :=TUzel(FSeznamVsechUzlu.Items[i]);
    If Uzel.Hrany.Count = 0 then
    Begin
    FSeznamVsechUzlu.Delete(i);
      If Uzel<>nil then
      Uzel.Free;
      Pozice:=i;
      Znovu:=True;
      Break;
    end;
   end;
   If Znovu then
   VymazUzly(Pozice);
   end;

   Function TVoronoiDiagram.Neexistuje(aPrusecik:TPrusecik;aLSP:TList):Boolean;
   var
   i:Integer;
   Begin
   Result:=True;
   For i:=0 to aLSP.Count-1 do
   Begin
   If TPrusecik(aLSP.Items[i]).Hrana = aPrusecik.Hrana then
   If TPrusecik(aLSP.Items[i]).X = aPrusecik.X then
   If TPrusecik(aLSP.Items[i]).Y = aPrusecik.Y then
   Result:=False;
   end;
   end;

   Procedure TVoronoiDiagram.ZvolVhodny(aNovyPrusecik,aExistujiciPrusecik:TPrusecik;aNovyGenerator,aNarusenyGenerator:TGenerator);
   var
   NovyP,ExistP:TPrusecik;
   HranaN,HranaE:THrana;
   VSU1,VSU2,VNU1,VNU2:Double;
   GS,GN:TGenerator;
   Begin

   NovyP:=aNovyPrusecik;
   ExistP:=aExistujiciPrusecik;
   HranaN:=NovyP.Hrana;
   HranaE:=ExistP.Hrana;
   GS:=aNarusenyGenerator;
   GN:=aNovyGenerator;
   VSU1:=DejVzdalenost(GS,HranaE.Uzel1);
   VSU2:=DejVzdalenost(GS,HranaE.Uzel2);
   VNU1:=DejVzdalenost(GN,HranaE.Uzel1);
   VNU2:=DejVzdalenost(GN,HranaE.Uzel2);

   If (VSU1+VSU2) <= (VNU1+VNU2) then
   Begin
   ExistP.Hrana:=HranaN;
   NovyP.Free;
   end;
   end;

   Function TVoronoiDiagram.NajdiDruhouHranu(aUzel:TUzel;aHrana:THrana;aStaryGenerator:TGenerator):THrana;
   var
   Uzel:TUzel;
   Generator:TGenerator;
   i:integer;
   Hrana:THrana;
   Begin
   Result:=Nil;
   Uzel:=aUzel;
   Hrana:=aHrana;
   Generator:=aStaryGenerator;

   For i:=0 to Uzel.Hrany.Count-1 do
   Begin
   If THrana(Generator.Hrany.Items[i]) <> Hrana then
   Begin
   Result:=THrana(Generator.Hrany.Items[i]);
   exit;
   end
   else
   Result:=NIL;
   end;
   end;

  Procedure TVoronoiDiagram.VygumujHranu(aHrana:THrana);
  var
  Hrana:THrana;
  U1,U2:TUzel;
  Begin
  Hrana:=aHrana;
  U1:=Hrana.Uzel1;
  U2:=Hrana.Uzel2;

  Form1.PaintBox1.Canvas.Pen.Width:=1;
  Form1.PaintBox1.Canvas.Pen.Style:=Graphics.psDot;
   Form1.PaintBox1.Canvas.Pen.Color:=ClSilver;
  Form1.PaintBox1.Canvas.MoveTo(Round(U1.X),Round(U1.Y));
  Form1.PaintBox1.Canvas.LineTo(Round(U2.X),Round(U2.Y));
  end;

  Procedure TVoronoiDiagram.PrekresliHrany;
  var
  i:integer;
  Hrana:THrana;
  Begin

  For i:=0 to FSeznamVsechHran.Count-1 do
  Begin
  Hrana:=THrana(FSeznamVsechHran.Items[i]);
  VykresliHranici(Hrana);
  end;
  end;

   Procedure TVoronoiDiagram.PrekresliVsechnyGeneratory;
   var
   i:integer;
   G:TGenerator;
   Begin
   For i:=0 to FSeznamVsechGeneratoru.Count-1 do
   Begin
   G:=TGenerator(FSeznamVsechGeneratoru.Items[i]);
   If G <> nil then
   Begin
   VykresliGenerator(G);
   end;
   end;
   end;

  Function TVoronoiDiagram.ReadPocet:Integer;
  Begin
  Result:=FSeznamVsechGeneratoru.Count;
  end;

  Procedure TVoronoiDiagram.PrekresliStartBod;
  Begin
  If StartBod = nil then exit;
  Form1.PaintBox1.Canvas.Pen.Width:=3;
  Form1.PaintBox1.Canvas.MoveTo(Round(StartBod.X),Round(StartBod.Y));
  Form1.PaintBox1.Canvas.Pen.Color:=ClLime;
  Form1.PaintBox1.Canvas.Ellipse(Round(StartBod.X)+PrumerStartaKonc,Round(StartBod.Y)+PrumerStartaKonc,Round(StartBod.X)-PrumerStartaKonc,Round(StartBod.Y)-PrumerStartaKonc);
  end;


  Procedure TVoronoiDiagram.PrekresliCilBod;
  Begin
  If CilBod = nil then exit;
  Form1.PaintBox1.Canvas.Pen.Width:=3;
  Form1.PaintBox1.Canvas.MoveTo(Round(CilBod.X),Round(CilBod.Y));
  Form1.PaintBox1.Canvas.Pen.Color:=ClRed;
  Form1.PaintBox1.Canvas.Ellipse(Round(CilBod.X)+PrumerStartaKonc,Round(CilBod.Y)+PrumerStartaKonc,Round(CilBod.X)-PrumerStartaKonc,Round(CilBod.Y)-PrumerStartaKonc);
  end;


   Function TVoronoiDiagram.NajdiPrusecik(aHranaVD:THrana;aUzel1,aUzel2:TUzel):TPrusecik;
   var
   HranaVD:THrana;
   KVD,K,SX,SY,qVD,q,PX,PY:Double;
   MaxX,MaxY,MinX,MinY:Double;
   MaxXHranice,MaxYHranice,MinXHranice,MinYHranice:Double;
   Uzel1,Uzel2:TUzel;
   Begin
   HranaVD:=aHranaVD;
   Result:=nil;
   Uzel1:=aUzel1;
   Uzel2:=aUzel2;

   If Uzel2.X = Uzel1.X then
   K:=Nekonecno;

   If HranaVD.Uzel2.X <>  HranaVD.Uzel1.X then
   KVD:=(HranaVD.Uzel2.Y-HranaVD.Uzel1.Y)/(HranaVD.Uzel2.X-HranaVD.Uzel1.X)
   else
   KVD:=Nekonecno;

   If Uzel2.X <> Uzel1.X then
   K:=(Uzel2.Y-Uzel1.Y)/(Uzel2.X-Uzel1.X)
   else
   K:=Nekonecno;

   IF K = KVD then
   Begin
   Result:=nil;
   exit;
   end;

   If KVD <> Nekonecno then
   Begin
   SX:=((HranaVD.Uzel2.X-HranaVD.Uzel1.X)/2)+HranaVD.Uzel1.X;
   SY:=((HranaVD.Uzel2.Y-HranaVD.Uzel1.Y)/2)+HranaVD.Uzel1.Y;
   qVD:=SY-(KVD*SX);
   end;

   If K <> Nekonecno then
   Begin
   SX:=((Uzel2.X-Uzel1.X)/2)+Uzel1.X;
   SY:=((Uzel2.Y-Uzel1.Y)/2)+Uzel1.Y;
   q:=SY-(K*SX);
   end;

   If KVD = Nekonecno then
   Begin
   PX:=HranaVD.Uzel1.X;
   PY:=PX*K+q;
   end;

   If K = Nekonecno then
   Begin
   PX:=Uzel1.X;
   PY:=PX*KVD+qVD
   end;

   If (K <> Nekonecno) and (KVD <> Nekonecno) then
   Begin
   PX:=(q-qVD)/(KVD-K);
   PY:=PX*K+q;
   end;

   MaxX:=Max(Uzel1.X,Uzel2.X);
   MinX:=Min(Uzel1.X,Uzel2.X);
   MaxY:=Max(Uzel1.Y,Uzel2.Y);
   MinY:=Min(Uzel1.Y,Uzel2.Y);

   MaxXHranice:=Max(aHranaVD.Uzel1.X,aHranaVD.Uzel2.X);
   MaxYHranice:=Max(aHranaVD.Uzel1.Y,aHranaVD.Uzel2.Y);
   MinXHranice:=Min(aHranaVD.Uzel1.X,aHranaVD.Uzel2.X);
   MinYHranice:=Min(aHranaVD.Uzel1.Y,aHranaVD.Uzel2.Y);

   If (PX <= MaxX) and (PX >= MinX) and (PY <= MaxY) and (PY >= MinY) then
   If (PX <= MaxXHranice) and (PX >= MinXHranice) and (PY <= MaxYHranice) and (PY >= MinYHranice) then
   Result:=TPrusecik.Create(PX,PY,HranaVD);
   end;

   Function TVoronoiDiagram.ExistPrusecik(aX,aY:Double;aPruseciky:TList):Boolean;
   var
    Pruseciky:TList;
    X,Y:Double;
    i:Integer;
   Begin
    Pruseciky:=aPruseciky;
    X:=aX;
    Y:=aY;
    Result:=False;
    For i:=0 to Pruseciky.Count-1 do
    If (TPrusecik(Pruseciky.Items[i]).X = X) and (TPrusecik(Pruseciky.Items[i]).Y = Y) then
    Result:=True;
   end;

   Procedure TVoronoiDiagram.ModifikujHranu(aHrana:THrana;aUzel:TUzel;aPrusecik:TPrusecik);
     var
     U,U1,U2:TUzel;
     P:TPrusecik;
     Hrana,Hranice:THrana;
     G1,G2:TGenerator;
     j,k:integer;
     Begin
      Hrana:=aHrana;
      U:=aUzel;
      P:=aPrusecik;
      G1:=TGenerator(Hrana.Generator1);
      G2:=TGenerator(Hrana.Generator2);
      U1:=TUzel(Hrana.Uzel1);
      U2:=TUzel(Hrana.Uzel2);

       For j:=0 to U1.Hrany.Count-1 do
        If Hrana = THrana(U1.Hrany.Items[j]) then
        Begin
        U1.Hrany.Delete(j);
        break;
        end;

        For j:=0 to U2.Hrany.Count-1 do
        If Hrana = THrana(U2.Hrany.Items[j]) then
        Begin
        U2.Hrany.Delete(j);
        break;
        end;

        If Hrana.Generator1 <> nil then
        Begin
        For j:=0 to G1.Hrany.Count-1 do
        If Hrana= THrana(G1.Hrany.Items[j]) then
        Begin
        G1.Hrany.Delete(j);
        break;
        end;
        end;

        If Hrana.Generator2 <> nil then
        Begin
        For j:=0 to G2.Hrany.Count-1 do
        If Hrana= THrana(G2.Hrany.Items[j]) then
        Begin
        G2.Hrany.Delete(j);
        break;
        end;
        end;

        For j:=0 to FSeznamVsechHran.Count-1 do
        If Hrana = THrana(FSeznamVsechHran.Items[j]) then
        Begin
        FSeznamVsechHran.Delete(j);
        break;
        end;

        VygumujHranu(Hrana);
        Hrana.Free;
        VytvorHranici(G1,G2,U,P);
     end;

   Function TVoronoiDiagram.Blizsi(aU1X,aU1Y:Double;aP1,aP2:TPrusecik):Boolean;
   var
    RX,RY:Double;
    VzdU1P1,VzdU1P2:Double;
    RX2,RY2,Soucet:Double;
    P1,P2:TPrusecik;
    U1X,U1Y:Double;
    Begin
    P1:=aP1;
    P2:=aP2;
    U1X:=aU1X;
    U1Y:=aU1Y;
    Result:=False;
    RX:=P1.X-U1X;
    RY:=P1.Y-U1Y;
    RX2:= Power(RX,2);
    RY2:= Power(RY,2);
    Soucet:=RX2+RY2;
    VzdU1P1:= Power(Soucet,0.5);

    RX:=P2.X-U1X;
    RY:=P2.Y-U1Y;
    RX2:= Power(RX,2);
    RY2:= Power(RY,2);
    Soucet:=RX2+RY2;
    VzdU1P2:= Power(Soucet,0.5);

    If VzdU1P1 < VzdU1P2 then
    Result:=True;
    end;

   Procedure TVoronoiDiagram.PropojBody;
   var
    NGS,NGC,G1,G2:TGenerator;
    SB,CB,U1,U2:TUzel;
    K,K1,K2:Double;
    i:integer;
    Hrana:THrana;
    Prusecik:TPrusecik;
    B:Boolean;
   Begin
    SB:=StartBod;
    CB:=CilBod;
    NGS:= NejblizsiGenerator(SB);
    NGC:= NejblizsiGenerator(CB);
    Prusecik:=nil;
    B:=False;

    If SB.X <> CB.X then
    K:=(CB.Y-SB.Y)/(CB.X-SB.X)
    else
    K:=Nekonecno;

    If SB.X <> NGS.X then
    K1:=(NGS.Y-SB.Y)/(NGS.X-SB.X)
    else
    K1:=Nekonecno;

   For i:=0 to NGS.Hrany.Count-1 do
   Begin
   Hrana:=THrana(NGS.Hrany.Items[i]);
   Prusecik:=NajdiPrusecik(Hrana,SB,CB);
   If Prusecik <> nil then
   Begin
   U1:=Hrana.Uzel1;
   U2:=Hrana.Uzel2;
   G1:=Hrana.Generator1;
   G2:=Hrana.Generator2;
   VytvorHranici(G1,G2,U1,Prusecik);
   VytvorHranici(G1,G2,Prusecik,U2);
   If ((G1 = NGS) and (G2 = NGC)) or ((G1 = NGC) and (G2 = NGS)) then
   Begin
   VymazHranu(Hrana);
   VytvorHranici(nil,nil,SB,Prusecik);
   VytvorHranici(nil,nil,CB,Prusecik);
   Break;
   end
   else
   Begin
   VymazHranu(Hrana);
   VytvorHranici(nil,nil,SB,Prusecik);
   B:=True;
   Break;
   end;
   end;
   end;

   If B then
   Begin
   For i:=0 to NGC.Hrany.Count-1 do
   Begin
   Hrana:=THrana(NGC.Hrany.Items[i]);
   Prusecik:=NajdiPrusecik(Hrana,SB,CB);
   If Prusecik <> nil then
   Begin
   U1:=Hrana.Uzel1;
   U2:=Hrana.Uzel2;
   G1:=Hrana.Generator1;
   G2:=Hrana.Generator2;
   VytvorHranici(G1,G2,U1,Prusecik);
   VytvorHranici(G1,G2,Prusecik,U2);
   VymazHranu(Hrana);
   Hrana:=VytvorHranici(nil,nil,CB,Prusecik);
   B:=False;
   Break;
   end;
   end;
   end;

   If NGS = NGC then
   Hrana:=VytvorHranici(nil,nil,SB,CB);

   For i:=FSeznamVsechHran.Count-1 downto 0 do
   If THrana(FSeznamVsechHran.Items[i]).Uzel1 = THrana(FseznamVsechHran.Items[i]).Uzel2 then
   VymazHranu(THrana(FseznamVsechHran.Items[i]));
   end;
end.
